
/* ==========================================================================
**
**                            ListEditor.c
**
** 1991 WILLISoft
**
** ==========================================================================
*/

#include "ListEditor.h"
#include "ListEditorClass.h"
#include "EmbossedGadgetClass.h"
#include <stdlib.h>
#include <string.h>
#include "minmax.h"
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include "amigamem.h"

#define STATE_ADDING  2
#define STATE_EDITING 1
#define STATE_NOTHING 0

#define BROWSE &self->browse
#define EDIT   &self->edit
#define ADD    &self->add
#define DELETE &self->delete


   void ListEditor_CleanUp( ListEditor *self )
   {
      CleanUp( BROWSE );
      CleanUp( EDIT );
      CleanUp( ADD );
      CleanUp( DELETE );
   }

   USHORT ListEditor_nGadgets( ListEditor *self )
   {
      return (USHORT) nGadgets( BROWSE ) + nGadgets( EDIT )
         + nGadgets( ADD ) + nGadgets( DELETE );
   }

   Gadget *ListEditor_FirstGadget( ListEditor *self )
   {
      return FirstGadget( BROWSE );
   }

   #define ADD_X 60
   #define ADD_Y 14
   #define Y_DIVIDER 2

   Point ListEditor_AskSize( ListEditor *self,
                             PIXELS      Width,
                             PIXELS      Height )
   {
      Point size, min_size, browse_size, edit_size;
      PIXELS browse_y;

      browse_size = MinSize( BROWSE );
      edit_size   = MinSize( EDIT );

      min_size.x = MAX( browse_size.x, edit_size.x );
      min_size.x = MAX( min_size.x, ADD_X*2 );
      min_size.y = browse_size.y + Y_DIVIDER + edit_size.y + Y_DIVIDER + ADD_Y;

      size.x = MAX( min_size.x, Width );
      size.y = MAX( min_size.y, Height );

      /* At this point we know that 'size' is at least the minimum size,
      ** but there may be more size constraints than that.
      ** (ScrollingLists currently must be an even font size.)
      */

      browse_y    = size.y - ADD_Y - Y_DIVIDER - edit_size.y - Y_DIVIDER;
      browse_size = AskSize( BROWSE, size.x, browse_y );

      size.x  = browse_size.x;
      size.y  = browse_size.y + Y_DIVIDER + edit_size.y + Y_DIVIDER + ADD_Y;

      return size;
   }


   Point ListEditor_SetSize( ListEditor *self,
                             PIXELS      Width,
                             PIXELS      Height )
   {
      Point size, edit_size;
      PIXELS browse_y;

      size = AskSize( self, Width, Height );

      edit_size = SetSize( EDIT, size.x, 0 );
      browse_y  = size.y - ADD_Y - Y_DIVIDER - edit_size.y - Y_DIVIDER;

      SetSize( BROWSE, size.x, browse_y );

      self->slist.Size = size;
      SetLocation( self, self->slist.Location.x, self->slist.Location.y );

      return size;
   }

   Point ListEditor_SetLocation( ListEditor *self,
                                 PIXELS      LeftEdge,
                                 PIXELS      TopEdge )
   {
      Point size, location, browse_size, edit_size;

      size        = Size( self );
      browse_size = Size( BROWSE );
      edit_size   = Size( EDIT );

      SetLocation( BROWSE, LeftEdge, TopEdge );
      SetLocation( EDIT, LeftEdge, TopEdge + browse_size.y + Y_DIVIDER );

      SetLocation( ADD,    LeftEdge,                TopEdge + size.y-ADD_Y);
      SetLocation( DELETE, LeftEdge + size.x-ADD_X, TopEdge + size.y-ADD_Y );

      location.x           = LeftEdge;
      location.y           = TopEdge;
      self->slist.Location = location;

      return location;
   }



   void ListEditor_Render( ListEditor  *self,
                           RastPort       *RPort )
   {
      Render( BROWSE, RPort );
      Render( EDIT,   RPort );
      Render( ADD,    RPort );
      Render( DELETE, RPort );
   }


   const StringList *ListEditor_StringList_of( ListEditor *self )
   {
      return StringList_of( BROWSE );
   }

   void ListEditor_AddString( ListEditor  *self,
                              char        *entry,
                              UBYTE        qualifier )
   {
      AddString( BROWSE, entry, qualifier );
   }


   void ListEditor_DeleteString( ListEditor  *self,
                                 USHORT       entry )
   {
      StringList *slist;

      slist = StringList_of(self);
      DeleteString( BROWSE, entry );
   }

   void ListEditor_DeleteAllStrings( ListEditor  *self )
   {
      StringList *slist;
      short i;

      DeleteAllStrings( BROWSE );
   }


   void ListEditor_SelectString( ListEditor  *self,
                                 USHORT       entry,
                                 BOOL         Select   )
   {
      SelectString( BROWSE, entry, Select );
   }


   void ListEditor_Refresh( ListEditor *self )
   {
      Refresh( BROWSE );
      Refresh( EDIT );
      Refresh( ADD );
      Refresh( DELETE );
   }

   USHORT ListEditor_ClaimEvent(  ListEditor     *self,
                                  IntuiMessage   *event )
   {
      return ClaimEvent( BROWSE, event )
           | ClaimEvent( EDIT,   event )
           | ClaimEvent( ADD,    event )
           | ClaimEvent( DELETE, event );
   }


   #define CLAIM_EVENT(b,e) (&b->g == (Gadget *) e->IAddress)

   USHORT ListEditor_Respond( ListEditor   *self,
                              IntuiMessage *event )
   {
      StringList  *slist;
      USHORT       i;
      char        *string, *val;

      USHORT browse_response  =0;
      USHORT edit_response    =0;
      USHORT add_response     =0;
      USHORT delete_response  =0;

      slist = StringList_of(self);
      if ((browse_response = Respond(BROWSE,event)) & CHANGED_STATE)
      {
         for (i=0; i<slist->nEntries; i++)
         {
            if (slist->Qualifiers[i] & ENTRY_SELECTED)
            {
               EnableIactor( EDIT, TRUE );
               EnableIactor( DELETE, TRUE );
               SetStringValue( EDIT, slist->Entries[i] );
               self->s     = i;
               self->state = STATE_EDITING;
               Refresh(EDIT);
               Refresh(DELETE);
               Activate(EDIT,TRUE);
               break;
            }
         }
      }
      else if ((edit_response = Respond(EDIT,event)) & DEACTIVATED)
      {
         val = StringValue( EDIT );
         if (self->state == STATE_ADDING)
         {
            if (strlen(val) >0)
            {
               AddString( self, val, 0 );
            }
         }
         else if (self->state == STATE_EDITING)
         {
            if (strlen(val) > 0)
            {
               if (string = Astrdup(val))
               {
                  Afree( slist->Entries[self->s] );
                  slist->Entries[self->s] = string;
               }
            }
            else
            {
               DeleteString( self, self->s );
            }
         }
         SetStringValue( EDIT, NULL );
         EnableIactor( EDIT, FALSE );
         EnableIactor( DELETE, FALSE );
         self->state = STATE_NOTHING;
         edit_response |= CHANGED_STATE;
         SelectString( self, self->s, FALSE );
         Refresh(EDIT);
         Refresh(DELETE);
         Refresh(BROWSE);
      }
      else if ((add_response = Respond(ADD,event)) & CHANGED_STATE)
      {
         if (self->state == STATE_EDITING)
         {
            SelectString( self, self->s, FALSE );
            Refresh( BROWSE );
         }
         self->state = STATE_ADDING;
         EnableIactor( EDIT, TRUE );
         SetStringValue( EDIT, NULL );
         Refresh(EDIT);
         Activate(EDIT,TRUE);
      }

      else if ((delete_response = Respond(DELETE,event)) & CHANGED_STATE)
      {
         self->state = STATE_NOTHING;
         DeleteString( self, self->s );
         EnableIactor( DELETE, FALSE );
         SetStringValue( EDIT, NULL ); 
         EnableIactor( EDIT, FALSE );
         Refresh(self);
      }

      return (USHORT) 
         browse_response | edit_response | add_response | delete_response;

   }

   BOOL ListEditor_Activate( ListEditor *self, BOOL activate )
   {
      return Activate( EDIT, activate );
   }

   void ListEditor_SetInteractorWindow( ListEditor *self,
                                        pcgWindow  *window )
   {
      self->slist.IaWindow = window;
      SetInteractorWindow( BROWSE, window );
      SetInteractorWindow( ADD,    window );
      SetInteractorWindow( EDIT,   window );
      SetInteractorWindow( DELETE, window );
   }

   ULONG ListEditor_IDCMPFlags( ListEditor *self )
   {
      return IDCMPFlags( BROWSE ) | IDCMPFlags( EDIT )
       | IDCMPFlags( ADD ) | IDCMPFlags( DELETE );
   }

   BOOL ListEditor_EnableIactor( ListEditor *self, BOOL enable )
   {
      if (enable)
      {
         EnableIactor( BROWSE, TRUE );
         EnableIactor( EDIT,   (self->state));
         EnableIactor( ADD,    TRUE );
         EnableIactor( DELETE, (self->state==STATE_EDITING) );
      }
      else
      {
         EnableIactor( BROWSE, FALSE );
         EnableIactor( EDIT,   FALSE );
         EnableIactor( ADD,    FALSE );
         EnableIactor( DELETE, FALSE );
      }
      return enable;
   }

   char *ListEditor_Title( ListEditor *self )
   {
      return Title( EDIT );
   }

   BOOL ListEditor_SetTitle( ListEditor *self, char *title )
   {
      return SetTitle( EDIT, title );
   }


   void ListEditor_Init( ListEditor *self,
                         PIXELS      LeftEdge,
                         PIXELS      TopEdge,
                         PIXELS      Width,
                         PIXELS      Height,
                         pcg_3DPens  pens,
                         char       *label )
   {

      StringLister_Init( &self->slist );

      self->slist.isa = ListEditorClass();
      self->state     = STATE_NOTHING;
      self->s         = 0;


      ScrollingList_Init( BROWSE, 0,0,0,0, pens, FALSE );
      StringGadget_Init( EDIT, 0,0,0, 80, pens, label );
      BoolGadget_Init( ADD, 0, 0, ADD_X, ADD_Y, pens, "Add" );
      BoolGadget_Init( DELETE, 0, 0, ADD_X, ADD_Y, pens, "Delete" );

      ChainGadgets( FirstGadget( BROWSE ), FirstGadget( EDIT )   );
      ChainGadgets( FirstGadget( EDIT ),   FirstGadget( ADD )    );
      ChainGadgets( FirstGadget( ADD ),    FirstGadget( DELETE ) );
      SetLocation( self, LeftEdge, TopEdge );
      SetSize( self, Width, Height );

      EnableIactor( self, TRUE );
   }


   #ifdef BUILDER
   #include "BuilderMethods.h"
   #include "GraphicObject_Builder.h"
   #include "ListEditor_coder.h"
   #include "ScrollingList_builder.h"
   #include "StringListerIO.h"

   ListEditor *ListEditor_New( ListEditor *self )
   {
      ListEditor *new_le;
      Point loc, size;
      StringList *sl1, *sl2;

      if (new_le = (ListEditor*) Amalloc(sizeof(ListEditor)))
      {
         if (self)
         {
            loc  = Location(self);
            size = Size(self);

            ListEditor_Init( new_le, loc.x, loc.y, size.x, size.y, 
                     self->add.Pens,
                  Title(self) );

            sl1 = StringList_of(self);
            sl2 = StringList_of(new_le);
            StringList_Dup( sl1, sl2 );
         }
         else
            ListEditor_Init( new_le, 0, 0, 128, 64, StandardPens(), NULL );

         GiveItAName(new_le);
         return new_le;
      }
      return NULL;
   }


   long ListEditor_WriteIFF( GraphicObject *obj, FILE *file )
   {
      return WriteStringListerChunk( obj, file, LSED );
   }

   GraphicObject* ListEditor_ReadIFF( Class            *class, 
                                      FILE             *file, 
                                      struct ChunkInfo *chunk )
   {
      return ReadStringListerChunk( class, file, chunk, LSED );
   }

   struct BuilderMethods ListEditor_bm;

   void ScrollingList_PropEdit();
   
   #endif

   BOOL ListEditor_elaborated = FALSE;

   struct StringListerClass ListEditor_Class;

   void ListEditorClass_Init( struct StringListerClass *class )
   {
      StringListerClass_Init( class );
      class->isa         = StringListerClass();
      class->CleanUp     = ListEditor_CleanUp;
      class->ClassName   = "ListEditor";
      class->AskSize     = ListEditor_AskSize;
      class->SetSize     = ListEditor_SetSize;
      class->SizeFlags   = GraphicObject_SizeFlagsAll;
      class->SetLocation = ListEditor_SetLocation;
      class->Title       = ListEditor_Title;
      class->SetTitle    = ListEditor_SetTitle;
      class->Respond     = ListEditor_Respond;
      class->Refresh     = ListEditor_Refresh;
      class->Render      = ListEditor_Render;
      class->ClaimEvent  = ListEditor_ClaimEvent;
      class->nGadgets    = ListEditor_nGadgets;
      class->FirstGadget = ListEditor_FirstGadget;
      class->SetInteractorWindow = ListEditor_SetInteractorWindow;
      class->IDCMPFlags  = ListEditor_IDCMPFlags;
      class->isEnabled   = EmbossedGadget_isEnabled;
      class->EnableIactor      = ListEditor_EnableIactor;
      class->Activate    = ListEditor_Activate;
      class->StringList_of = ListEditor_StringList_of;
      class->AddString     = ListEditor_AddString;
      class->DeleteString  = ListEditor_DeleteString;
      class->DeleteAllStrings = ListEditor_DeleteAllStrings;
      class->SelectString  = ListEditor_SelectString;

   #ifdef BUILDER
      go_InitBuilderMethods( &ListEditor_bm );

      ListEditor_bm.New       = ListEditor_New;
      ListEditor_bm.WriteCode = ListEditor_WriteCode;
      ListEditor_bm.PropEdit  = ScrollingList_PropEdit;
      ListEditor_bm.WriteIFF  = ListEditor_WriteIFF;
      ListEditor_bm.ReadIFF   = ListEditor_ReadIFF;
      class->BuilderMethods   = &ListEditor_bm;
   #endif
   }



   struct StringListerClass *ListEditorClass( void )
   {
      if (! ListEditor_elaborated)
      {
         ListEditorClass_Init( &ListEditor_Class );
         ListEditor_elaborated = TRUE;
      }

      return &ListEditor_Class;
   }


